-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimpleActuator
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "GameLib"
require "Window"
require "Unit"

-------------------------------------------------------------------------------------
-- SimpleActuator Module Definition
-----------------------------------------------------------------------------------------------
local SimpleActuator = {} 
local run = true

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimpleActuator:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimpleActuator:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {"ClassResources"}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- SimpleActuator OnLoad
-----------------------------------------------------------------------------------------------
function SimpleActuator:OnLoad()
	self:InitializeHooks()	
end

function SimpleActuator:InitializeHooks()
	local crb = Apollo.GetAddon("ClassResources")
	crb.OnMedicUpdateTimer = self.OnMedicUpdateTimer
	crb.OnMedicEnteredCombat = self.OnMedicEnteredCombat
end


function SimpleActuator:OnMedicUpdateTimer()
	local unitPlayer = GameLib.GetPlayerUnit()	-- Can instead just listen to a CharacterChange, CharacterCreate, etc. event
	local nResourceMax = unitPlayer:GetMaxResource(1)
	local nResourceCurrent = unitPlayer:GetResource(1)
	local tBuffs = unitPlayer:GetBuffs()

	-- Mana
	local nManaMax = math.floor(unitPlayer:GetMaxMana())
	local nManaCurrent = math.floor(unitPlayer:GetMana())
	self.tWindowMap["ManaProgressBar"]:SetMax(nManaMax)
	self.tWindowMap["ManaProgressBar"]:SetProgress(nManaCurrent)
	self.tWindowMap["ManaProgressBar"]:SetStyleEx("EdgeGlow", self.bCombat and (nManaCurrent / nManaMax < 0.97))
	self.tWindowMap["ManaProgressBar"]:SetTooltip(String_GetWeaselString(Apollo.GetString("MedicResource_FocusTooltip"), nManaCurrent, nManaMax))

	self.tWindowMap["ManaProgressText"]:SetText(nManaCurrent == nManaMax and "" or (math.floor(nManaCurrent / nManaMax * 100).."%"))
	self.tWindowMap["ManaProgressText"]:SetTextColor(self.bCombat and ApolloColor.new("UI_TextHoloTitle") or ApolloColor.new("ff56b381"))
	self.tWindowMap["ManaProgressBacker"]:Show(nManaCurrent ~= nManaMax)

	-- Partial Node Count
	local nPartialCount = 0
	for idx, tCurrBuffData in pairs(tBuffs.arBeneficial or {}) do
		if tCurrBuffData.splEffect:GetId() == 42569 then -- TODO replace with code enum
			nPartialCount = tCurrBuffData.nCount
			break
		end
	end

	-- Nodes
	local bFirstPartial = true
	for idx = 1, 4 do
		-- Full vs Partial
		local bInnate = GameLib.IsCurrentInnateAbilityActive()
		local strIndex = "MedicNode"..idx
		local bFull = nResourceCurrent >= idx
		local bShowPartial = bFirstPartial and nPartialCount > 0
		self.tWindowMap[strIndex..":FillSprite"]:Show(bFull or bShowPartial, not (bFull or bShowPartial))
		self.tWindowMap[strIndex..":EmptySprite"]:Show(not bFull)

		-- Anim
		local wndCurr = self.tWindowMap[strIndex]
		if bFull and not wndCurr:GetData() then
			wndCurr:SetSprite(self.bCombat and "sprMedic_Anim_BrightGreenGrow" or "sprMedic_Anim_OutOfCombatGreenGrow")
		elseif not bFull and wndCurr:GetData() then
			wndCurr:SetSprite(self.bCombat and "sprMedic_Anim_BrightGreenFade" or "sprMedic_Anim_OutOfCombatGreenFade")
		end
		wndCurr:SetData(bFull)

		-- Sprite -- Behaviour changed
		local strSpriteToUse = ""
		if not bInnate then
			strSpriteToUse = "CM_MedicSprites:sprMedic_Cube_DullGreen"
		else
			strSpriteToUse = "CM_MedicSprites:sprMedic_Cube_BrightGreen_3"
		end
		self.tWindowMap[strIndex..":FillSprite"]:SetSprite(strSpriteToUse)
	end

	
	self.tWindowMap["MedicBaseFrame_InCombat"]:SetSprite(nResourceCurrent == 0 and "sprMedic_Base_InCombatRed" or "sprMedic_Base_InCombatGreen")

	self:HelperToggleVisibiltyPreferences(self.wndMain, unitPlayer)
end

function SimpleActuator:OnMedicEnteredCombat(unitPlayer, bInCombat)
	if unitPlayer ~= GameLib.GetPlayerUnit() or not self.wndMain or not self.wndMain:IsValid() then
		return
	end

	self.bCombat = bInCombat

	if bInCombat then
		self.tWindowMap["BG_12Squares"]:Show(false, false, 3)
		self.tWindowMap["MedicBaseFrame_InCombat"]:Show(false, false, 3)
		self.wndMain:FindChild("MedicBaseFrame_OutOfCombat"):Show(false)
		self.tWindowMap["ManaProgressBar"]:SetFullSprite("sprMedic_ProgBar_InCombat")
	else
		self.tWindowMap["BG_12Squares"]:Show(false, false, 3)
		self.tWindowMap["MedicBaseFrame_InCombat"]:Show(false, false, 3)
		self.wndMain:FindChild("MedicBaseFrame_OutOfCombat"):Show(false)
		self.tWindowMap["ManaProgressBar"]:SetFullSprite("sprMedic_ProgBar_OutOfCombat")
	end
end

-----------------------------------------------------------------------------------------------
-- SimpleActuator Instance
-----------------------------------------------------------------------------------------------
local SimpleActuatorInst = SimpleActuator:new()
SimpleActuatorInst:Init()
